# These relations are symmetric: it doesn't matter which concept is their
# 'start' or 'end'.
SYMMETRIC_RELATIONS = {
    '/r/RelatedTo',
    '/r/SimilarTo',
    '/r/EtymologicallyRelatedTo',
    '/r/Synonym',
    '/r/Antonym',
    '/r/DistinctFrom',
}